@@ -306,7 +306,6 @@ on_session_open_failed (GdmSession      *session,
 
 static void
 on_session_session_started (GdmSession      *session,
-                            int              pid,
                             GdmFactorySlave *slave)
 {
         g_debug ("GdmFactorySlave: Relay session started");
@@ -482,6 +481,7 @@ run_greeter (GdmFactorySlave *slave)
         gboolean       display_is_local;
         char          *display_id;
         char          *display_name;
+        char          *seat_id;
         char          *display_device;
         char          *display_hostname;
         char          *auth_file;
@@ -492,6 +492,7 @@ run_greeter (GdmFactorySlave *slave)
         display_is_local = FALSE;
         display_id = NULL;
         display_name = NULL;
+        seat_id = NULL;
         auth_file = NULL;
         display_device = NULL;
         display_hostname = NULL;
@@ -500,6 +501,7 @@ run_greeter (GdmFactorySlave *slave)
                       "display-is-local", &display_is_local,
                       "display-id", &display_id,
                       "display-name", &display_name,
+                      "display-seat-id", &seat_id,
                       "display-hostname", &display_hostname,
                       "display-x11-authority-file", &auth_file,
                       NULL);
@@ -556,6 +558,7 @@ run_greeter (GdmFactorySlave *slave)
 
         g_debug ("GdmFactorySlave: Creating greeter on %s %s", display_name, display_device);
         slave->priv->greeter = gdm_greeter_session_new (display_name,
+                                                        seat_id,
                                                         display_device,
                                                         display_hostname,
                                                         display_is_local);
@@ -585,6 +588,7 @@ run_greeter (GdmFactorySlave *slave)
 
         g_free (display_id);
         g_free (display_name);
+        g_free (seat_id);
         g_free (display_device);
         g_free (display_hostname);
         g_free (auth_file);
@@ -683,7 +687,7 @@ gdm_factory_slave_run (GdmFactorySlave *slave)
                 if (! res) {
                         g_warning (_("Could not start the X "
                                      "server (your graphical environment) "
-                                     "due to some internal error. "
+                                     "due to an internal error. "
                                      "Please contact your system administrator "
                                      "or check your syslog to diagnose. "
                                      "In the meantime this display will be "
