@@ -234,7 +234,7 @@ write_pid (void)
         errno = 0;
         pf = open (GDM_PID_FILE, O_WRONLY|O_CREAT|O_TRUNC|O_EXCL, 0644);
         if (pf < 0) {
-                g_warning (_("Cannot write PID file %s: possibly out of diskspace: %s"),
+                g_warning (_("Cannot write PID file %s: possibly out of disk space: %s"),
                            GDM_PID_FILE,
                            g_strerror (errno));
 
@@ -247,7 +247,7 @@ write_pid (void)
         close (pf);
 
         if (written < 0) {
-                g_warning (_("Cannot write PID file %s: possibly out of diskspace: %s"),
+                g_warning (_("Cannot write PID file %s: possibly out of disk space: %s"),
                            GDM_PID_FILE,
                            g_strerror (errno));
                 return;
@@ -344,7 +344,6 @@ gdm_daemon_check_permissions (uid_t uid,
 {
         struct stat statbuf;
         const char *auth_path;
-        int         res;
 
         auth_path = LOGDIR;
 
@@ -354,12 +353,7 @@ gdm_daemon_check_permissions (uid_t uid,
         set_effective_user_group (0, 0);
 
         /* Now set things up for us as  */
-        res = chown (auth_path, 0, gid);
-        if (res == -1) {
-                g_warning ("Unable to change owner for auth file: %s",
-                           g_strerror (errno));
-        }
-
+        chown (auth_path, 0, gid);
         g_chmod (auth_path, (S_IRWXU|S_IRWXG|S_ISVTX));
 
         set_effective_user_group (uid, gid);
@@ -537,7 +531,7 @@ main (int    argc,
         static gboolean     fatal_warnings   = FALSE;
         static GOptionEntry entries []   = {
                 { "fatal-warnings", 0, 0, G_OPTION_ARG_NONE, &fatal_warnings, N_("Make all warnings fatal"), NULL },
-                { "timed-exit", 0, 0, G_OPTION_ARG_NONE, &do_timed_exit, N_("Exit after a time - for debugging"), NULL },
+                { "timed-exit", 0, 0, G_OPTION_ARG_NONE, &do_timed_exit, N_("Exit after a time (for debugging)"), NULL },
                 { "version", 0, 0, G_OPTION_ARG_NONE, &print_version, N_("Print GDM version"), NULL },
 
                 { NULL }
@@ -566,11 +560,6 @@ main (int    argc,
                 goto out;
         }
 
-        if (print_version) {
-                g_print ("GDM %s\n", VERSION);
-                exit (1);
-        }
-
         if (fatal_warnings) {
                 GLogLevelFlags fatal_mask;
 
