@@ -121,9 +121,7 @@ gdm_language_chooser_widget_add_language (GdmLanguageChooserWidget *widget,
                                              readable_language,
                                              0,
                                              FALSE,
-                                             FALSE,
-                                             NULL,
-                                             NULL);
+                                             FALSE);
                 g_free (language);
         }
         g_free (readable_language);
