@@ -50,6 +50,7 @@ struct GdmLanguageOptionWidgetPrivate
 
 enum {
         LANGUAGE_ACTIVATED,
+        DIALOG_HIDDEN,
         NUMBER_OF_SIGNALS
 };
 
@@ -94,6 +95,7 @@ gdm_language_option_widget_hide_dialog (GdmLanguageOptionWidget *widget)
 {
         gtk_widget_destroy (widget->priv->dialog);
         widget->priv->dialog = NULL;
+        g_signal_emit (G_OBJECT (widget), signals[DIALOG_HIDDEN], 0);
 }
 
 static void
@@ -170,6 +172,16 @@ gdm_language_option_widget_class_init (GdmLanguageOptionWidgetClass *klass)
                                                      G_TYPE_NONE,
                                                      0);
 
+        signals[DIALOG_HIDDEN] = g_signal_new ("dialog-hidden",
+                                               G_TYPE_FROM_CLASS (object_class),
+                                               G_SIGNAL_RUN_FIRST,
+                                               G_STRUCT_OFFSET (GdmLanguageOptionWidgetClass, dialog_hidden),
+                                               NULL,
+                                               NULL,
+                                               g_cclosure_marshal_VOID__VOID,
+                                               G_TYPE_NONE,
+                                               0);
+
         g_type_class_add_private (klass, sizeof (GdmLanguageOptionWidgetPrivate));
 }
 
@@ -228,7 +240,7 @@ gdm_language_option_widget_init (GdmLanguageOptionWidget *widget)
                                     /* translators: This brings up a dialog
                                      * with a list of languages to choose from
                                      */
-                                    C_("language", "Other..."),
+                                    C_("language", "Other…"),
                                     _("Choose a language from the "
                                       "full list of available languages."),
                                     GDM_OPTION_WIDGET_POSITION_BOTTOM);
@@ -259,7 +271,7 @@ gdm_language_option_widget_new (void)
         GObject *object;
 
         object = g_object_new (GDM_TYPE_LANGUAGE_OPTION_WIDGET,
-                               "label-text", _("Language"),
+                               "label-text", _("_Language:"),
                                "icon-name", "preferences-desktop-locale",
                                "max-item-count", 8,
                                NULL);
